<?php
/**
 * Implementation of hook_access().
 */
function family_group_access($op, $node) {
  if ($op == 'create' || $op == 'update' || $op == 'delete') {
    return user_access('add/edit/delete family');
  }
  if ($op == 'view') {
    return family_check_privacy($node->fid);
  }
}

/**
 * Implementation of hook_form().
 */
function family_group_form(&$node) {
  $form['title'] = array(
    '#type' => 'hidden',
	'#default_value' => $node->MARR,
  );
  $form['MARR']['MARR_DATE'] = array(
    '#type' => 'textfield',
    '#title' => t('Marriage Date'),
    '#default_value' => $node->MARR_DATE,
    '#required' => FALSE,
  );
  $form['MARR']['MARR_PLAC'] = array(
    '#type' => 'textfield',
    '#title' => t('Marriage Place'),
    '#default_value' => $node->MARR_PLAC,
    '#required' => FALSE,
  );
  $form['MARR']['MARR_TYPE'] = array(
    '#type' => 'textfield',
    '#title' => t('Marriage Type'),
    '#default_value' => $node->MARR_TYPE,
    '#required' => FALSE,
  );
  $form['DIV']['DIV_DATE'] = array(
    '#type' => 'textfield',
    '#title' => t('Divorce Date'),
    '#default_value' => $node->DIV_DATE,
    '#required' => FALSE,
  );
  $form['DIV']['DIV_PLAC'] = array(
    '#type' => 'textfield',
	'#title' => t('Divorce Place'),
	'#default_value' => $node->DIV_PLAC,
	'#required' => FALSE,
  );
  $form['body_filter']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('Notes'),
    '#default_value' => $node->body,
    '#rows' => 20,
    '#required' => FALSE
  );
  $form['body_filter']['format'] = filter_form($node->format); //Not sure why this goes here, but all the examples have it...
  return $form;
} // function family_group_form(&$node, &$param)

/**
 * Implementation of hook_insert().
 *  Insert All the facts and relationships
 *  Update Node and revision to replace data not in form
 *
 *  RELATIONSHIPS:  Base facts are related to FAM (fact, FAM)
 *                  Subfacts are related to super fact  (sub, super)
 *  NOTE:
 *  Should we generate the gedcom code at the same time,
 *  or leave that for exporting?
 */
function family_group_insert($node) {
  //Insert Group Line - TODO: Need to be able to assign a new xref & insert Gedcom code
  if ($node->fid) {
    $grp_fid=$node->fid;
  }
  else {
    $grp_fid = db_next_id('{family_facts}_fid');
  }
  db_query("INSERT INTO {family_facts} (fid, nid, xref, fact_code, fact_value,
                         gedcom_source) VALUES (%d, %d, '%s', '%s', '%s', '%s')", $grp_fid,
                        $node->nid, $node->xref,'FAM',$node->value,$node->gedcom_source);

  //INSERT MARRIAGE if any of date, place, or type exist
  if ($node->MARR_DATE or $node->MARR_PLACE or $node->MARR_TYPE){
    $marr_fid =  family_insert_fact('MARR', '');
    family_insert_relation($marr_fid, $grp_fid, 'FACT');
	// INSERT DATE & RELATION
	if ($node->MARR_DATE){
	  $fid = family_insert_fact('DATE', $node->MARR_DATE);
	  family_insert_relation($fid, $marr_fid, 'FACT');
	}
	// INSERT PLAC & RELATION
	if ($node->MARR_PLAC){
	  $fid = family_insert_fact('PLAC', $node->MARR_PLAC);
	  family_insert_relation($fid, $marr_fid, 'FACT');
	}
	// INSERT TYPE & RELATION
	if ($node->MARR_TYPE){
	  $fid = family_insert_fact('TYPE', $node->MARR_TYPE);
	  family_insert_relation($fid, $marr_fid, 'FACT');
	}
  }

  //INSERT DIV if either date or place exist
  if ($node->DIV_DATE or $node->DIV_PLAC){
    $div_fid = family_insert_fact('DIV', '');
	family_insert_relation($div_fid, $grp_fid, 'FACT');
	//INSERT DATE & RELATION
	if ($node->DIV_DATE){
	  $fid = family_insert_fact('DATE', $node->DIV_DATE);
	  family_insert_relation($fid, $div_fid, 'FACT');
	}
	//INSERT PLAC & RELATION
	if ($node->DIV_PLAC){
	  $fid = family_insert_fact('PLAC', $node->DIV_PLAC);
	  family_insert_relation($fid, $div_fid, 'FACT');
	}
  }
}

/**
 * Implementation of family_group_load().
 * This function will grab all the descriptive data for an group
 * Primary use is for filling in form data to preform an edit
 * [Code mostly taken from family_line_indi()]
 *
 * Returns an array of additions
 */
function family_group_load($node) {
  $nid = $node->nid;
  //If they don't exist then there is no point in going on so return the empty array
  $additions->fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE nid = %d",$nid));
  //MARR
  if ($marr_fid = family_get_subfid_by_code("MARR", $additions->fid)) {
    $marr_date = family_get_subvalue_by_code("DATE", $marr_fid);
	if ($marr_date) {
	  $additions->MARR_DATE = $marr_date;
	}
	$marr_plac = family_get_subvalue_by_code("PLAC", $marr_fid);
	if ($marr_plac) {
	  $additions->MARR_PLAC = $marr_plac;
	}
	$marr_type = family_get_subvalue_by_code("TYPE", $marr_fid);
	if ($marr_type) {
	  $additions->MARR_TYPE = $marr_plac;
	}
  }
  //DIV
  if ($div_fid = family_get_subfid_by_code("DIV", $additions->fid)) {
    $div_date = family_get_subvalue_by_code("DATE", $div_fid);
	if ($div_date) {
	  $additions->DIV_DATE = $div_date;
	}
	$div_plac = family_get_subvalue_by_code("PLAC", $div_fid);
	if ($div_plac) {
	  $additions->DIV_PLAC = $div_plac;
	}
  }
  return($additions);
}

/**
 * Implementation of hook_update().
 *  Update the facts and new relationships
 *  Update Node and revision to replace data not in form
 *
 *  See family_individual_insert
 */
function family_group_update($node) {
  //UPDATE MARR
  //If there is marriage data, we need to get it to update it
  if ($marr_fid = family_get_subfid_by_code('MARR', $node->fid)) {
    $marr_date_fid = family_get_subfid_by_code('DATE', $marr_fid);
	$marr_plac_fid = family_get_subfid_by_code('PLAC', $marr_fid);
	$marr_type_fid = family_get_subfid_by_code('TYPE', $marr_fid);
  }
  //If any marriage parts are listed, we need to make sure we have a MARR
  if (($node->MARR_DATE == '') && ($node->MARR_PLAC == '') && ($node->MARR_TYPE =='')) {
    family_remove_fact($marr_fid);
  }
  elseif (! $marr_fid) {
    $marr_fid = family_insert_fact('MARR', '');
	family_insert_relation($marr_fid, $node->fid,'FACT');
  }
  //UPDATE MARR DATE & RELATION
  //If the node has a date OR a date exists
  if ($marr_date_fid && ($node->MARR_DATE == '') ){ family_remove_fact($marr_date_fid); }
  elseif ($marr_date_fid) {
    family_update_fact($marr_date_fid, $node->MARR_DATE);
  }
  elseif ($node->MARR_DATE != '') {
    $fid = family_insert_fact('DATE', $node->MARR_DATE);
    family_insert_relation($fid, $marr_fid, 'FACT');
  }
  //UPDATE MARR PLAC & RELATION
  if ($marr_plac_fid && ($node->MARR_PLAC == '') ){ family_remove_fact($marr_plac_fid); }
  elseif ($marr_plac_fid) {
    family_update_fact($marr_plac_fid, $node->MARR_PLAC);
  }
  elseif ($node->MARR_PLAC != '') {
    $fid = family_insert_fact('PLAC', $node->MARR_PLAC);
    family_insert_relation($fid, $marr_fid, 'FACT');
  }
  //UPDATE MARR TYPE & RELATION
  if ($marr_type_fid && ($node->MARR_TYPE == '') ){ family_remove_fact($marr_type_fid); }
  elseif ($marr_type_fid) {
    family_update_fact($marr_type_fid, $node->MARR_TYPE);
  }
  elseif ($node->MARR_TYPE != '') {
    $fid = family_insert_fact('PLAC', $node->MARR_TYPE);
    family_insert_relation($fid, $marr_fid, 'FACT');
  }
  //UPDATE DIV
  //If there is divorce data, we need to get it to update it
  if ($div_fid = family_get_subfid_by_code('DIV', $node->fid)) {
    $div_date_fid = family_get_subfid_by_code('DATE', $div_fid);
	$div_plac_fid = family_get_subfid_by_code('PLAC', $div_fid);
  }
  //If any divorce parts are listed, we need to make sure we have a MARR
  if (($node->DIV_DATE == '') && ($node->DIV_PLAC == '')) {
    family_remove_fact($div_fid);
  }
  elseif (! $div_fid) {
    $div_fid = family_insert_fact('DIV', '');
	family_insert_relation($div_fid, $node->fid,'FACT');
  }
  //UPDATE DIV DATE & RELATION
  //If the node has a date OR a date exists
  if ($div_date_fid && ($node->DIV_DATE == '') ){ family_remove_fact($div_date_fid); }
  elseif ($div_date_fid) {
    family_update_fact($div_date_fid, $node->DIV_DATE);
  }
  elseif ($node->DIV_DATE != '') {
    $fid = family_insert_fact('DATE', $node->DIV_DATE);
    family_insert_relation($fid, $div_fid, 'FACT');
  }
  //UPDATE MARR PLAC & RELATION
  if ($div_plac_fid && ($node->DIV_PLAC == '') ){ family_remove_fact($div_plac_fid); }
  elseif ($div_plac_fid) {
    family_update_fact($div_plac_fid, $node->DIV_PLAC);
  }
  elseif ($node->DIV_PLAC != '') {
    $fid = family_insert_fact('PLAC', $node->DIV_PLAC);
    family_insert_relation($fid, $div_fid, 'FACT');
  }
}

/**
 * Implementation of hook_view().
 */
function family_group_view(&$node, $teaser = FALSE, $page = FALSE) {
  $node = node_prepare($node, $teaser);
//  $fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE nid= %d", $node->nid));
//  $node->body = family_view_indi($node->fid) . $node->body;
//  $node->teaser = family_view_indi($node->fid) . $node->teaser;
  $node->content['family_group_view'] = array(
      '#value' => theme($teaser? 'family_group_teaser':'family_group_body', $node),
      '#weight' => 0,
  );

  return $node;
}

function theme_family_group_teaser($node) {
  return family_view_grou($node->fid);
}
function theme_family_group_body($node) {
  return family_view_grou($node->fid);
}
/**
 * Implementation of hook_delete().
 */
function family_group_delete(&$node) {
//  $ind_fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE nid = %d",$node->nid));
  family_remove_fact($node->fid);
}